"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@graphback/core");
const resolverTemplates_1 = require("./resolverTemplates");
function createMutations(modelType, crudOptions) {
    const mutations = {};
    const modelName = modelType.name;
    if (crudOptions.create) {
        const fieldName = core_1.getFieldName(modelName, core_1.GraphbackOperationType.CREATE);
        mutations[fieldName] = resolverTemplates_1.createTemplate(modelName);
    }
    if (crudOptions.update) {
        const fieldName = core_1.getFieldName(modelName, core_1.GraphbackOperationType.UPDATE);
        mutations[fieldName] = resolverTemplates_1.updateTemplate(modelName);
    }
    if (crudOptions.delete) {
        const fieldName = core_1.getFieldName(modelName, core_1.GraphbackOperationType.DELETE);
        mutations[fieldName] = resolverTemplates_1.deleteTemplate(modelName);
    }
    return mutations;
}
exports.createMutations = createMutations;
function createQueries(modelType, crudOptions) {
    const queries = {};
    const modelName = modelType.name;
    if (crudOptions.find) {
        const fieldName = core_1.getFieldName(modelName, core_1.GraphbackOperationType.FIND);
        queries[fieldName] = resolverTemplates_1.findTemplate(modelName);
    }
    if (crudOptions.findAll) {
        const fieldName = core_1.getFieldName(modelName, core_1.GraphbackOperationType.FIND_ALL);
        queries[fieldName] = resolverTemplates_1.findAllTemplate(modelName);
    }
    return queries;
}
exports.createQueries = createQueries;
function createSubscriptions(modelType, crudOptions) {
    const subscriptions = {};
    const modelName = modelType.name;
    if (crudOptions.create && crudOptions.subCreate) {
        const fieldName = core_1.getSubscriptionName(modelName, core_1.GraphbackOperationType.CREATE);
        subscriptions[fieldName] = resolverTemplates_1.newSubscriptionTemplate(modelName);
    }
    if (crudOptions.update && crudOptions.subUpdate) {
        const fieldName = core_1.getSubscriptionName(modelName, core_1.GraphbackOperationType.UPDATE);
        subscriptions[fieldName] = resolverTemplates_1.updatedSubscriptionTemplate(modelName);
    }
    if (crudOptions.delete && crudOptions.subDelete) {
        const fieldName = core_1.getSubscriptionName(modelName, core_1.GraphbackOperationType.DELETE);
        subscriptions[fieldName] = resolverTemplates_1.deletedSubscriptionTemplate(modelName);
    }
    return subscriptions;
}
exports.createSubscriptions = createSubscriptions;
function generateCRUDResolversFunctions(models) {
    const outputResolvers = { Query: {}, Mutation: {}, Subscription: {} };
    for (const { graphqlType, crudOptions } of models) {
        const newQueries = createQueries(graphqlType, crudOptions);
        outputResolvers.Query = Object.assign(Object.assign({}, outputResolvers.Query), newQueries);
        const newMutations = createMutations(graphqlType, crudOptions);
        outputResolvers.Mutation = Object.assign(Object.assign({}, outputResolvers.Mutation), newMutations);
        const newSubs = createSubscriptions(graphqlType, crudOptions);
        outputResolvers.Subscription = Object.assign(Object.assign({}, outputResolvers.Subscription), newSubs);
    }
    return outputResolvers;
}
exports.generateCRUDResolversFunctions = generateCRUDResolversFunctions;
//# sourceMappingURL=createResolvers.js.map