"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const formatter_1 = require("./formatter");
//Wraps resolver method strings into fully featured files
const topNote = `/**
* File generated by Graphback CRUD resolver plugin.
* Content will be overwritten by Graphback generator.
* To change implementation please disable generator options and supply your own implementation
* outside generated file.
*/`;
exports.resolverFileTemplateTS = (outputResolvers, options) => {
    return `${topNote}\n\nexport default {
      ${outputResolvers.join('\n\n\t')}  
    }`;
};
exports.resolverFileTemplateJS = (outputResolvers, options) => {
    return `${topNote}\n\n exports = {
      ${outputResolvers.join('\n\n')}  
    }`;
};
const mapResolverKeyValueTemplates = (resolvers) => {
    const resolverNames = Object.keys(resolvers);
    return resolverNames.map((resolverName) => {
        const resolverFn = resolvers[resolverName];
        return `${resolverName}: ${resolverFn}`;
    });
};
function createResolverFileTemplate(outputResolvers, options) {
    switch (options.format) {
        case 'js':
            return formatter_1.formatDocumentJS(exports.resolverFileTemplateJS(outputResolvers, options));
        case 'ts':
            return formatter_1.formatDocumentTs(exports.resolverFileTemplateTS(outputResolvers, options));
        default:
            throw new Error(`"${options.format}" resolvers are not supported`);
    }
}
exports.createResolverTemplate = (typeResolvers, options) => {
    const mutations = mapResolverKeyValueTemplates(typeResolvers.Mutation);
    const queries = mapResolverKeyValueTemplates(typeResolvers.Query);
    const subscriptions = mapResolverKeyValueTemplates(typeResolvers.Subscription);
    const outputResolvers = [];
    if (options.layout === "apollo") {
        if (queries.length) {
            outputResolvers.push(`Query: {
            ${queries.join(',\n')}
        },`);
        }
        if (mutations.length) {
            outputResolvers.push(`Mutation: {
            ${mutations.join(',\n')}
        },`);
        }
        if (subscriptions.length) {
            outputResolvers.push(`Subscription: {
            ${subscriptions.join(',\n')}
        }`);
        }
    }
    else if (options.layout === "graphql") {
        outputResolvers.push(`${queries.join(',\n')}`);
        outputResolvers.push(`${mutations.join(',\n')}`);
        outputResolvers.push(`${subscriptions.join(',\n')}`);
    }
    else {
        throw new Error("Wrong layout specified in resolver generator plugin");
    }
    return createResolverFileTemplate(outputResolvers, options);
};
//# sourceMappingURL=resolverWrapper.js.map