"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@graphback/core");
const formatter_1 = require("./formatter");
function generatePubSubContext(models) {
    return models.reduce((template, model) => {
        const objName = model.graphqlType.name;
        const lowercasedName = core_1.lowerCaseFirstChar(objName);
        const chunk = `
        const ${lowercasedName}PubSubContext = {
            pubSub,
            publishCreate: ${model.crudOptions.subCreate},
            publishUpdate: ${model.crudOptions.subUpdate},
            publishDelete: ${model.crudOptions.subDelete}
        }`;
        return template + chunk;
    }, '');
}
function generateContextObject(models) {
    return models.reduce((template, model) => {
        const objName = model.graphqlType.name;
        const lowercasedName = core_1.lowerCaseFirstChar(objName);
        const chunk = `
            ${objName}: createKnexCRUDRuntimeContext('${objName}', schema, db, ${lowercasedName}PubSubContext),
        `;
        return template + chunk;
    }, '');
}
function tsTemplate(models) {
    return `
    import { createKnexCRUDRuntimeContext, KnexRuntimeContextConfig } from "@graphback/runtime"
    
    export const createCRUDResolversRuntimeContext = (options: KnexRuntimeContextConfig) => {
      const { schema, db, pubSub } = options;
      ${generatePubSubContext(models)}
    
      return {
        ${generateContextObject(models)}
      }
    }
    `;
}
function jsTemplate(models) {
    return `
    const { createKnexCRUDRuntimeContext, KnexRuntimeContextConfig } = require('@graphback/runtime');
    
    module.exports.createCRUDResolversRuntimeContext = (options) => {
        const { schema, db, pubSub } = options;
        ${generatePubSubContext(models)}
    
        return {
        ${generateContextObject(models)}
        }
    }
    `;
}
exports.createRuntimeFile = (models, config) => {
    let output;
    if (config.format === 'js') {
        output = jsTemplate(models);
        output = formatter_1.formatDocumentJS(output);
    }
    else {
        output = tsTemplate(models);
        output = formatter_1.formatDocumentTs(output);
    }
    return {
        fileName: `createContext.${config.format}`,
        output
    };
};
//# sourceMappingURL=runtimeTemplate.js.map