import { GraphbackCoreMetadata, GraphbackPlugin } from '@graphback/core';
import { OutputFileSystem } from './GeneratorModel';
import { GeneratorResolversFormat } from './GeneratorResolversFormat';
export interface ResolverGeneratorPluginConfig {
    format: 'ts' | 'js';
    /**
     * RelativePath for the output files created by generator
     */
    outputPath: string;
    /**
     * Name of the generated resolvers file (default: resolvers.(format))
     */
    resolversFileName?: string;
    /**
     * Layout of the of the resolvers object.
     * Supports Apollo (GraphQL-Tools) or GraphQL reference spec format
     */
    layout?: 'apollo' | 'graphql';
}
/**
 * Graphback Resolver File Generator Plugin
 *
 * Generates:
 *
 * - default CRUD resolvers for all model types.
 * - blank resolver files to implements custom resolvers
 * - CRUD Services for each type for controling the data sources
 *
 * And then writes to content to resolver files in the server.
 *
 */
export declare class ResolverGeneratorPlugin extends GraphbackPlugin {
    private pluginConfig;
    constructor(pluginConfig: ResolverGeneratorPluginConfig);
    getPluginName(): string;
    createResources(metadata: GraphbackCoreMetadata): void;
    generateResolvers(metadata: GraphbackCoreMetadata): OutputFileSystem;
    protected createGeneratedResolversFile(resolvers: GeneratorResolversFormat, pluginConfig: ResolverGeneratorPluginConfig): {
        fileName: string;
        output: string;
    };
}
